@@ -5,25 +5,23 @@
 #include <cassert>
 #include <cstdio>
 
-#include "download/download_main.h"
 #include "manager.h"
+#include "download/download_main.h"
 #include "net/throttle_list.h"
 #include "protocol/extensions.h"
 #include "protocol/handshake_manager.h"
 #include "torrent/download_info.h"
 #include "torrent/error.h"
 #include "torrent/exceptions.h"
-#include "torrent/poll.h"
 #include "torrent/throttle.h"
-#include "torrent/tracker/dht_controller.h"
+#include "torrent/net/fd.h"
+#include "torrent/net/network_config.h"
+#include "torrent/net/network_manager.h"
+#include "torrent/net/poll.h"
 #include "torrent/utils/log.h"
 #include "utils/diffie_hellman.h"
 #include "utils/sha1.h"
 
-#define LT_LOG(log_fmt, ...)                                            \
-  lt_log_print(LOG_CONNECTION_HANDSHAKE, "handshake->%s: " log_fmt,     \
-               m_address.pretty_address_str().c_str(), __VA_ARGS__);
-
 #if DISABLED__USE_EXTRA_DEBUG
 #define LT_LOG_EXTRA_DEBUG_SA(sa, log_fmt, ...)                         \
   lt_log_print(LOG_CONNECTION_HANDSHAKE, "handshake->%s: " log_fmt, sap_pretty_str(m_address).c_str(), __VA_ARGS__);
@@ -51,7 +49,8 @@ private:
 } // namespace
 
 namespace torrent {
-Handshake::Handshake(SocketFd fd, HandshakeManager* m, int encryptionOptions) :
+
+Handshake::Handshake(int fd, HandshakeManager* m, int encryptionOptions) :
   m_manager(m),
 
   // Use global throttles until we know which download it is.
@@ -61,7 +60,7 @@ Handshake::Handshake(SocketFd fd, HandshakeManager* m, int encryptionOptions) :
   m_encryption(encryptionOptions),
   m_extensions(torrent::HandshakeManager::default_extensions()) {
 
-  set_fd(fd);
+  set_fd(SocketFd(fd));
 
   m_readBuffer.reset();
   m_writeBuffer.reset();
@@ -81,7 +80,7 @@ Handshake::initialize_incoming(const sockaddr* sa) {
   m_incoming = true;
   m_address = sa_copy(sa);
 
-  if (m_encryption.options() & (ConnectionManager::encryption_allow_incoming | ConnectionManager::encryption_require))
+  if (m_encryption.options() & (net::NetworkConfig::encryption_allow_incoming | net::NetworkConfig::encryption_require))
     m_state = READ_ENC_KEY;
   else
     m_state = READ_INFO;
@@ -123,7 +122,6 @@ Handshake::deactivate_connection() {
   m_state = INACTIVE;
 
   this_thread::scheduler()->erase(&m_task_timeout);
-
   this_thread::poll()->remove_and_close(this);
 }
 
@@ -164,12 +162,12 @@ Handshake::destroy_connection() {
 
 int
 Handshake::retry_options() {
-  uint32_t options = m_encryption.options() & ~ConnectionManager::encryption_enable_retry;
+  uint32_t options = m_encryption.options() & ~net::NetworkConfig::encryption_enable_retry;
 
   if (m_encryption.retry() == HandshakeEncryption::RETRY_PLAIN)
-    options &= ~ConnectionManager::encryption_try_outgoing;
+    options &= ~net::NetworkConfig::encryption_try_outgoing;
   else if (m_encryption.retry() == HandshakeEncryption::RETRY_ENCRYPTED)
-    options |= ConnectionManager::encryption_try_outgoing;
+    options |= net::NetworkConfig::encryption_try_outgoing;
   else
     throw internal_error("Invalid retry type.");
 
@@ -224,7 +222,7 @@ Handshake::read_encryption_key() {
 
     if (m_readBuffer.peek_8() == 19 && std::memcmp(m_readBuffer.position() + 1, m_protocol, 19) == 0) {
       // got unencrypted BT handshake
-      if (m_encryption.options() & ConnectionManager::encryption_require)
+      if (m_encryption.options() & net::NetworkConfig::encryption_require)
         throw handshake_error(ConnectionManager::handshake_dropped, e_handshake_unencrypted_rejected);
 
       m_state = READ_INFO;
@@ -320,6 +318,10 @@ Handshake::read_encryption_skey() {
 
   validate_download();
 
+  // We don't allow encrypted connections for meta-data downloads.
+  if (m_download->info()->is_meta_download())
+    throw handshake_error(ConnectionManager::handshake_dropped, e_handshake_invalid_encryption);
+
   std::make_pair(m_uploadThrottle, m_downloadThrottle) = m_download->throttles(m_address.get());
 
   m_encryption.initialize_encrypt(m_download->info()->hash().c_str(), m_incoming);
@@ -364,10 +366,10 @@ Handshake::read_encryption_negotiation() {
 
   // choose one of the offered encryptions, or check the chosen one is valid
   if (m_incoming) {
-    if ((m_encryption.options() & ConnectionManager::encryption_prefer_plaintext) && m_encryption.has_crypto_plain()) {
+    if ((m_encryption.options() & net::NetworkConfig::encryption_prefer_plaintext) && m_encryption.has_crypto_plain()) {
       m_encryption.set_crypto(HandshakeEncryption::crypto_plain);
 
-    } else if ((m_encryption.options() & ConnectionManager::encryption_require_RC4) && !m_encryption.has_crypto_rc4()) {
+    } else if ((m_encryption.options() & net::NetworkConfig::encryption_require_RC4) && !m_encryption.has_crypto_rc4()) {
       throw handshake_error(ConnectionManager::handshake_dropped, e_handshake_unencrypted_rejected);
 
     } else if (m_encryption.has_crypto_rc4()) {
@@ -389,7 +391,7 @@ Handshake::read_encryption_negotiation() {
     if (m_encryption.crypto() != HandshakeEncryption::crypto_rc4 && m_encryption.crypto() != HandshakeEncryption::crypto_plain)
       throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_encryption);
 
-    if ((m_encryption.options() & ConnectionManager::encryption_require_RC4) && (m_encryption.crypto() != HandshakeEncryption::crypto_rc4))
+    if ((m_encryption.options() & net::NetworkConfig::encryption_require_RC4) && (m_encryption.crypto() != HandshakeEncryption::crypto_rc4))
       throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_encryption);
   }
 
@@ -613,7 +615,7 @@ Handshake::read_port() {
   m_readBuffer.read_8();
 
   if (length == 2)
-    manager->dht_controller()->add_node(m_address.get(), m_readBuffer.peek_16());
+    runtime::network_manager()->dht_add_peer_node(m_address.get(), m_readBuffer.peek_16());
 
   m_readBuffer.consume(length);
   return true;
@@ -889,16 +891,21 @@ Handshake::validate_download() {
 
 void
 Handshake::event_write() {
+  int socket_error;
+
   try {
 
     switch (m_state) {
     case CONNECTING:
-      if (get_fd().get_error())
+      if (!fd_get_socket_error(m_fileDesc, &socket_error))
+        throw internal_error("Handshake::event_write() fd_get_socket_error failed : " + std::string(strerror(errno)));
+
+      if (socket_error != 0)
         throw handshake_error(ConnectionManager::handshake_failed, e_handshake_network_unreachable);
 
       this_thread::poll()->insert_read(this);
 
-      if (m_encryption.options() & ConnectionManager::encryption_use_proxy) {
+      if (m_encryption.options() & net::NetworkConfig::encryption_use_proxy) {
         prepare_proxy_connect();
 
         m_state = PROXY_CONNECT;
@@ -915,11 +922,11 @@ Handshake::event_write() {
 
       m_writeBuffer.reset();
 
-      if (m_encryption.options() & (ConnectionManager::encryption_try_outgoing | ConnectionManager::encryption_require)) {
+      if (m_encryption.options() & (net::NetworkConfig::encryption_try_outgoing | net::NetworkConfig::encryption_require)) {
         prepare_key_plus_pad();
 
         // if connection fails, peer probably closed it because it was encrypted, so retry encrypted if enabled
-        if (!(m_encryption.options() & ConnectionManager::encryption_require))
+        if (!(m_encryption.options() & net::NetworkConfig::encryption_require))
           m_encryption.set_retry(HandshakeEncryption::RETRY_PLAIN);
 
         m_state = READ_ENC_KEY;
@@ -1019,7 +1026,7 @@ Handshake::prepare_enc_negotiation() {
   HandshakeEncryption::copy_vc(m_writeBuffer.end());
   m_writeBuffer.move_end(HandshakeEncryption::vc_length);
 
-  if (m_encryption.options() & ConnectionManager::encryption_require_RC4)
+  if (m_encryption.options() & net::NetworkConfig::encryption_require_RC4)
     m_writeBuffer.write_32(HandshakeEncryption::crypto_rc4);
   else
     m_writeBuffer.write_32(HandshakeEncryption::crypto_plain | HandshakeEncryption::crypto_rc4);
@@ -1038,9 +1045,14 @@ Handshake::prepare_handshake() {
   m_writeBuffer.write_range(m_protocol, m_protocol + 19);
 
   std::memset(m_writeBuffer.end(), 0, 8);
-  *(m_writeBuffer.end()+5) |= 0x10;    // support extension protocol
-  if (manager->dht_controller()->is_active())
-    *(m_writeBuffer.end()+7) |= 0x01;  // DHT support, enable PORT message
+
+  // Supports extension protocol.
+  *(m_writeBuffer.end()+5) |= 0x10;
+
+  // Send PORT message.
+  if (runtime::network_manager()->is_dht_active())
+    *(m_writeBuffer.end()+7) |= 0x01;
+
   m_writeBuffer.move_end(8);
 
   m_writeBuffer.write_range(m_download->info()->hash().c_str(), m_download->info()->hash().c_str() + 20);
@@ -1105,15 +1117,10 @@ Handshake::prepare_post_handshake(bool must_write) {
   Buffer::iterator old_end = m_writeBuffer.end();
 
   // Send PORT message for DHT if enabled and peer supports it.
-  if (m_peerInfo->supports_dht() &&
-      manager->dht_controller()->is_active() &&
-      manager->dht_controller()->is_receiving_requests()) {
-
+  if (m_peerInfo->supports_dht() && runtime::network_manager()->is_dht_active_and_receiving_requests()) {
     m_writeBuffer.write_32(3);
     m_writeBuffer.write_8(protocol_port);
-    m_writeBuffer.write_16(manager->dht_controller()->port());
-
-    // manager->dht_controller()->port_sent();
+    m_writeBuffer.write_16(runtime::network_manager()->dht_port());
   }
 
   // Send a keep-alive if we still must send something.
